@@ -30,12 +30,12 @@
 // alphas of a stable release, typically the nightly builds. 
 // Most of the time we're in development, so IS_ALPHA should be defined
 // to 1.
-#define IS_ALPHA 0   // Turn it off for beta release.
+#define IS_ALPHA 0
 
 // Increment as appropriate every time you release a new version
 #define AUDACITY_VERSION   1
 #define AUDACITY_RELEASE   3
-#define AUDACITY_REVISION  13
+#define AUDACITY_REVISION  14
 #define AUDACITY_MODLEVEL  0
 
 #if IS_ALPHA
