@@ -422,11 +422,7 @@ void FreqWindow::GetAudio()
    
    if (selcount == 0)
       return;
-   
-   if (selcount > 1)
-      for(i=0; i<mDataLen; i++)
-         mBuffer[i] /= selcount;
-   
+
    if (warning) {
       wxString msg;
       msg.Printf(_("Too much audio was selected.  Only the first %.1f seconds of audio will be analyzed."),
@@ -1307,15 +1303,3 @@ void FreqPlot::OnMouseEvent(wxMouseEvent & event)
 {
    freqWindow->PlotMouseEvent(event);
 }
-
-// Indentation settings for Vim and Emacs and unique identifier for Arch, a
-// version control system. Please do not modify past this point.
-//
-// Local Variables:
-// c-basic-offset: 3
-// indent-tabs-mode: nil
-// End:
-//
-// vim: et sts=3 sw=3
-// arch-tag: 8fce6648-17b9-4632-b845-3a6d25117783
-
